﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Administration
{
    enum MessageTargets
    {
        GAME, 
        MAIL,
    }

    enum ItemOrigins
    {
        BASE,
        SHIP,
    }

    /// <summary>
    /// Požadavek na určitou operaci od uživatele
    /// </summary>
    abstract class Request
    {
        protected ProtocolWriter protocolWriter;

        /// <summary>
        /// Vytvoří řetězcovou reprezentaci požadavku podle platného protokolu
        /// </summary>
        /// <returns>Řetězcová reprezentace požadavku</returns>
        public abstract string ToMessageString();

        /// <summary>
        /// Druh požadavku zprávy
        /// </summary>
        public enum RequestNames
        {
            UNDEFINED,
            BAN_PLAYER,
            GET_PLAYER_DETAILS,
            GET_PLAYERS_LIST,
            GENERATE_PASSWORD,
            SEND_MESSAGE,
            GET_SHIP_DETAILS, 
            GET_BASE_DETAILS,
            DELETE_SHIP_FROM_PLAYER,
            DELETE_BASE_FROM_PLAYER,
            DELETE_ITEM_FROM_SHIP,
            DELETE_ITEM_FROM_BASE,
        }

        /// <summary>
        /// Druh požadavku zprávy
        /// </summary>
        public abstract RequestNames Name
        {
            get;
        }

        /// <summary>
        /// Vytvoří novou instanci třídy <c>Request</c>
        /// </summary>
        public Request()
        {
            protocolWriter = new ProtocolWriter();
        }
    }

    /// <summary>
    /// Požadavek na generování nového hesla pro hráče
    /// </summary>
    class GeneratePasswordRequest : Request
    {
        /// <summary>
        /// ID hráče, pro kterého se má generovat nové heslo
        /// </summary>
        private ulong playerID;

        public override RequestNames Name
        {
            get
            {
                return RequestNames.GENERATE_PASSWORD;
            }
        }

        /// <summary>
        /// Vytvoří novou instanci třídy <c>GeneratePasswordRequest</c>
        /// </summary>
        /// <param name="playerID">ID hráče, pro kterého se má generovat nové heslo</param>
        public GeneratePasswordRequest(ulong playerID)
        {
            this.playerID = playerID;
        }

        public override string ToMessageString()
        {
            protocolWriter.beginMessage(MessageType.MT_REQUEST, Name.ToString());
            protocolWriter.writeValue("PLAYER_ID", playerID);

            protocolWriter.endMessage(true);
            return protocolWriter.copyMessage();
        }
    }

    /// <summary>
    /// Požadavek na zaslání zprávy pro hráče
    /// </summary>
    class SendMessageRequest : Request
    {
        /// <summary>
        /// ID hráče, kterému se má zaslat zpráva
        /// </summary>
        private ulong playerID;

        /// <summary>
        /// Cílové prostředí, do kterého se má zaslat zpráva
        /// </summary>
        private MessageTargets target;

        /// <summary>
        /// Skutečné tělo zasílané zprávy
        /// </summary>
        private string message;

        /// <summary>
        /// Předmět zasílané zprávy
        /// </summary>
        private string subject;

        public override RequestNames Name
        {
            get
            {
                return RequestNames.SEND_MESSAGE;
            }
        }

        /// <summary>
        /// Vytvoří novou instanci třídy <c>SendMessageRequest</c>
        /// </summary>
        /// <param name="playerID">ID hráče, kterému se má zaslat zpráva</param>
        /// <param name="subject">Předmět zasílané zprávy</param>
        /// <param name="message">Skutečné tělo zasílané zprávy</param>
        /// <param name="target">Cílové prostředí, do kterého se má zaslat zpráva</param>
        public SendMessageRequest(ulong playerID, string subject, string message, MessageTargets target)
        {
            this.playerID = playerID;
            this.subject = subject;
            this.message = message;
            this.target = target;
        }

        public override string ToMessageString()
        {
            protocolWriter.beginMessage(MessageType.MT_REQUEST, Name.ToString());
            protocolWriter.writeValue("PLAYER_ID", playerID);

            protocolWriter.endMessage(true);
            return protocolWriter.copyMessage();
        }
    }

    /// <summary>
    /// Požadavek na získání podrobností o konkrétní základně
    /// </summary>
    class GetBaseDetailsRequest : Request
    {
        /// <summary>
        /// ID základny, pro kterou chceme získat podrobnosti
        /// </summary>
        private ulong baseID;

        public override RequestNames Name
        {
            get
            {
                return RequestNames.GET_SHIP_DETAILS;
            }
        }

        /// <summary>
        /// Vytvoří novou instanci třídy <c>GetBaseDetailsRequest</c>
        /// </summary>
        /// <param name="baseID">ID základny, pro kterou chceme získat podrobnosti</param>
        public GetBaseDetailsRequest(ulong baseID)
        {
            this.baseID = baseID;
        }

        public override string ToMessageString()
        {
            protocolWriter.beginMessage(MessageType.MT_REQUEST, Name.ToString());
            protocolWriter.writeValue("BASE_ID", baseID);

            protocolWriter.endMessage(true);
            return protocolWriter.copyMessage();
        }
    }

    /// <summary>
    /// Požadavek na odstranění lodi ze seznamu hráčových lodí
    /// </summary>
    class DeleteShipRequest : Request
    {
        /// <summary>
        /// ID lodě, kterou chceme odstranit
        /// </summary>
        private ulong shipID;

        public override RequestNames Name
        {
            get
            {
                return RequestNames.DELETE_SHIP_FROM_PLAYER;
            }
        }

        /// <summary>
        /// Vytvoří novou instanci třídy <c>DeleteShipRequest</c>
        /// </summary>
        /// <param name="shipID">ID lodě, kterou chceme odstranit</param>
        public DeleteShipRequest(ulong shipID)
        {
            this.shipID = shipID;
        }

        public override string ToMessageString()
        {
            protocolWriter.beginMessage(MessageType.MT_REQUEST, Name.ToString());
            protocolWriter.writeValue("SHIP_ID", shipID);

            protocolWriter.endMessage(true);
            return protocolWriter.copyMessage();
        }
    }

    /// <summary>
    /// Požadavek na odstranění základny ze seznamu hráčových základen
    /// </summary>
    class DeleteBaseRequest : Request
    {
        /// <summary>
        /// ID základny, kterou chceme odstranit
        /// </summary>
        private int baseID;

        public override RequestNames Name
        {
            get
            {
                return RequestNames.DELETE_BASE_FROM_PLAYER;
            }
        }

        /// <summary>
        /// Vytvoří novou instanci třídy <c>DeleteBaseRequest</c>
        /// </summary>
        /// <param name="shipID">ID základny, kterou chceme odstranit</param>
        public DeleteBaseRequest(int baseID)
        {
            this.baseID = baseID;
        }

        public override string ToMessageString()
        {
            protocolWriter.beginMessage(MessageType.MT_REQUEST, Name.ToString());
            protocolWriter.writeValue("BASE_ID", baseID);

            protocolWriter.endMessage(true);
            return protocolWriter.copyMessage();
        }
    }

    /// <summary>
    /// Požadavek na odstranění základny ze seznamu hráčových základen
    /// </summary>
    class DeleteItemRequest : Request
    {
        /// <summary>
        /// ID předmětu, který chceme odstranit
        /// </summary>
        private ulong itemID;

        /// <summary>
        /// Původ předmětu, odkud se má odstranit
        /// </summary>
        private ItemOrigins origin;

        public override RequestNames Name
        {
            get
            {
                if (origin == ItemOrigins.BASE)
                {
                    return RequestNames.DELETE_ITEM_FROM_BASE;
                }
                else if (origin == ItemOrigins.SHIP)
                {
                    return RequestNames.DELETE_ITEM_FROM_SHIP;
                }
                else
                {
                    return RequestNames.UNDEFINED;
                }
            }
        }

        /// <summary>
        /// Vytvoří novou instanci třídy <c>DeleteItemRequest</c>
        /// </summary>
        /// <param name="shipID">ID předmětu, který chceme odstranit</param>
        /// /// <param name="origin">Původ předmětu, odkud se má odstranit</param>
        public DeleteItemRequest(ulong itemID, ItemOrigins origin)
        {
            this.itemID = itemID;
        }

        public override string ToMessageString()
        {
            protocolWriter.beginMessage(MessageType.MT_REQUEST, Name.ToString());
            protocolWriter.writeValue("ITEM_ID", itemID);

            protocolWriter.endMessage(true);
            return protocolWriter.copyMessage();
        }
    }

    [System.AttributeUsage(
        AttributeTargets.Class | AttributeTargets.Field | AttributeTargets.Struct)
    ]
    public class ProtocolMapping : System.Attribute
    {       
        private string name;

        public ProtocolMapping(string name)
        {
            this.name = name;
        }

        public ProtocolMapping()
        {
            this.name = string.Empty;
        }

        public String Name
        {
            get
            {
                return name;
            }
        }
    }

    [ProtocolMapping("ping")]
    class PingRequest
    {        
    }

    [ProtocolMapping("big")]
    class BigRequest
    {
        public class InnerClass
        {
            public int ahoj;
            public string nazdar;
        }

        public InnerClass[] inner = new InnerClass[2];
    }

    [ProtocolMapping("big")]
    class BigRequest2
    {
        public class InnerClass
        {
            public int ahoj;
            public string nazdar;
        }

        public InnerClass inner = new InnerClass();
    }

    [ProtocolMapping(Requests.REQUEST_ADMIN_PING)]
    public struct PingAnswer
    {
    }

    [ProtocolMapping(Requests.REQUEST_ADMIN_LOGIN)]
    public struct LoginRequest
    {
        public string player_login;
        public string player_password;
    }

    [ProtocolMapping(Requests.REQUEST_ADMIN_LOGIN)]
    public struct LoginAnswer
    {
    }

    public struct PlayerInformation
    {
        public ulong player_id;

        public string player_name;

        public string player_login;

        public bool player_has_ban;
    }

    /// <summary>
    /// Požadavek na získání seznamu hráčů s odpovídajícím jménem
    /// </summary>
    [ProtocolMapping(Requests.REQUEST_ADMIN_PLAYERS)]
    public struct PlayersRequest
    {
        /// <summary>
        /// Jméno hráče, nebo část jména těch hráčů, které chceme v seznamu získat
        /// </summary>
        public string player_login_filter;
    };

    [ProtocolMapping(Requests.REQUEST_ADMIN_PLAYERS)]
    public struct PlayersAnswer
    {
        public PlayerInformation[] player;
    };

    public struct ShipInformation
    {
        public ulong fortress_id;     
                                                               
        public string fortress_name; 
                                                                
        public string fortress_class;
    }

    /// <summary>
    /// Požadavek na získání podrobností o konkrétním hráči
    /// </summary>
    [ProtocolMapping(Requests.REQUEST_ADMIN_PLAYER_INFORMATION)]
    public struct PlayerInformationRequest
    {
        /// <summary>
        /// ID hráče, pro kterého chceme získat podrobnosti
        /// </summary>
        public ulong player_id;
    }

    [ProtocolMapping(Requests.REQUEST_ADMIN_PLAYER_INFORMATION)]
    public struct PlayerInformationAnswer
    {
        public PlayerInformation player;
        
        [ProtocolMapping("ship")]
        public ShipInformation[] ship_information;
        
        [ProtocolMapping("base")]
        public ShipInformation[] base_information;
    }

    [ProtocolMapping(Requests.REQUEST_ADMIN_CREATE_SHIP)]
    public struct CreateShipRequest
    {
        public ulong player_id;
    }

    [ProtocolMapping(Requests.REQUEST_ADMIN_CREATE_SHIP)]
    public struct CreateShipAnswer
    {
        public ulong player_id;

        public ShipInformation created_ship;

        public bool ship_created;
    }

    /// <summary>
    /// Požadavek na získání podrobností o konkrétní lodi
    /// </summary>
    [ProtocolMapping(Requests.REQUEST_ADMIN_SHIP_INFORMATION)]
    public class ShipInformationRequest
    {
        /// <summary>
        /// ID lodě, pro kterou chceme získat podrobnosti
        /// </summary>
        public ulong fortress_id;
    }

    /// <summary>
    /// Odpověď na získání podrobností o konkrétní lodi
    /// </summary>
    [ProtocolMapping(Requests.REQUEST_ADMIN_SHIP_INFORMATION)]
    public class ShipInformationAnswer
    {
        /// <summary>
        /// ID lodě, pro kterou chceme získat podrobnosti
        /// </summary>
        public ulong fortress_id;
        public string fortress_name;
        public string fortress_class;
        public double coordinate_x;
        public double coordinate_y;
    }

    /// <summary>
    /// Požadavek na získání podrobností o konkrétní základně
    /// </summary>
    [ProtocolMapping(Requests.REQUEST_ADMIN_BASE_INFORMATION)]
    public class BaseInformationRequest
    {
        /// <summary>
        /// ID lodě, pro kterou chceme získat podrobnosti
        /// </summary>
        public ulong fortress_id;
    }

    /// <summary>
    /// Odpověď na získání podrobností o konkrétní základně
    /// </summary>
    [ProtocolMapping(Requests.REQUEST_ADMIN_BASE_INFORMATION)]
    public class BaseInformationAnswer
    {
        /// <summary>
        /// ID pevnosti, pro kterou chceme získat podrobnosti
        /// </summary>
        public ulong fortress_id;
        public string fortress_name;
        public string fortress_class;
        public double coordinate_x;
        public double coordinate_y;
    }

    /// <summary>
    /// Požadavek na zabanování hráče
    /// </summary>
    [ProtocolMapping(Requests.REQUEST_ADMIN_BAN_PLAYER)]
    public class BanPlayerRequest 
    {
        /// <summary>
        /// ID zabanovaného hráče
        /// </summary>
        public ulong player_id;

        /// <summary>
        /// Počet dní, na které hráč dostane ban
        /// </summary>
        public int ban_duration_in_days;

        /// <summary>
        /// Úroveň uděleného banu
        /// </summary>
        public int ban_level;

        /// <summary>
        /// Důvod pro zabanování hráče
        /// </summary>
        public string ban_reason;

        /// <summary>
        /// Zda ban existuje pro daného hráče
        /// </summary>
        public bool player_has_ban;
    }

    /// <summary>
    /// Odpověď na zabanování hráče
    /// </summary>
    [ProtocolMapping(Requests.REQUEST_ADMIN_BAN_PLAYER)]
    public class BanPlayerAnswer
    {
        /// <summary>
        /// ID zabanovaného hráče
        /// </summary>
        public ulong player_id;

        /// <summary>
        /// Výsledek akce banování
        /// </summary>
        public string ban_result;

        /// <summary>
        /// Zda ban existuje pro daného hráče
        /// </summary>
        public bool player_has_ban;
    }

    /// <summary>
    /// Požadavek na zaslání informací o  banu hráče
    /// </summary>
    [ProtocolMapping(Requests.REQUEST_ADMIN_PLAYER_BAN_INFORMATION)]
    public class PlayerBanInformationRequest 
    {
        /// <summary>
        /// ID hráče
        /// </summary>
        public ulong player_id;
    }

    [ProtocolMapping(Requests.REQUEST_ADMIN_PLAYER_BAN_INFORMATION)]
    public class PlayerBanInformationAnswer 
    {
        /// <summary>
        /// ID zabanovaného hráče
        /// </summary>
        public ulong player_id;

        /// <summary>
        /// Počet dní, na které hráč dostane ban
        /// </summary>
        public int ban_duration_in_days;

        /// <summary>
        /// Úroveň uděleného banu
        /// </summary>
        public int ban_level;

        /// <summary>
        /// Důvod pro zabanování hráče
        /// </summary>
        public string ban_reason;

        /// <summary>
        /// Zda ban existuje pro daného hráče
        /// </summary>
        public bool player_has_ban;
    }

    [ProtocolMapping(Requests.REQUEST_ADMIN_ADD_REALM)]
    public class AddRealmRequest
    {
        public string username;
        public string realname;
        public string password;

        public int ship_count;
        public int base_count;

        public double ship_quality;
        public double base_quality;
        public double science_quality;

        public double[] ships;
        public double[] bases;
    }

    [ProtocolMapping(Requests.REQUEST_ADMIN_ADD_REALM)]
    public class AddRealmAnswer
    {
        public string player_adding_result;
    }
}
