#include "BuildWidget.h"
#include <cassert>
#include "softProj\settings.h"
#include "softProj\WeaponsWidget\fightWidgetRowWidget.h"
#include "../streams/ErrorStream.h"

extern KODIAK_DEBUG_STREAM KODIAK_DEBUG_COUT;

BuildWidgetTable::BuildWidgetTable(BuildWidget * parent, EFortressBuildTableType tableType) : QWidget(parent) {
    titleLabel_ = 0;
    tableWidget_ = 0;
    searchButton_ = 0;
    searchEdit_ = 0;

    tradeWidget_ = parent;	
	this->tableType_ = tableType;

	clickedFortressRow_ = NULL;
}

void BuildWidgetTable::setTitle(QPushButton * titleLabel) {
    this->titleLabel_ = titleLabel;
}

void BuildWidgetTable::setTable(QTableWidget * tableWidget) {
    this->tableWidget_ = tableWidget;
	tableWidget_->verticalHeader()->setResizeMode(QHeaderView::Interactive);
}

void BuildWidgetTable::setSearchButton(QPushButton * searchButton) {
    this->searchButton_ = searchButton;
}

void BuildWidgetTable::setShowAllButton(QPushButton * showAllButton) {
    this->showAllButton_ = showAllButton;
}

void BuildWidgetTable::setSearchEdit(QLineEdit * searchEdit) {
    this->searchEdit_ = searchEdit;
}

void BuildWidgetTable::connectSignals() {
    bool isConnected = connect(searchButton_, SIGNAL(clicked()), this, SLOT(searchClickedSlot()));
    assert(isConnected);

    isConnected = connect(showAllButton_, SIGNAL(clicked()), this, SLOT(showAllClickedSlot()));
    assert(isConnected);

    isConnected = connect(tableWidget_, SIGNAL(cellClicked(int,int)), this, SLOT(cellClickedSlot(int, int)));
    assert(isConnected);    
}

void BuildWidgetTable::showAllClickedSlot() {
    clearFilteredContent();

	for (std::map<long, FightWidgetRow *>::const_iterator it = rows_.begin(); it != rows_.end(); ++it) {
		insertItemIntoTable(it->second);
    }
}

void BuildWidgetTable::searchClickedSlot() {
    QString searchedString = searchEdit_->text();
    QRegExp rx(".*" + searchedString + ".*", Qt::CaseInsensitive);

    clearFilteredContent();

    ItemsVector matchedItems;

	for (std::map<long, FightWidgetRow *>::const_iterator it = rows_.begin(); it != rows_.end(); ++it) {
		if (rx.exactMatch(it->second->getNameQString())) {
            matchedItems.push_back(it->second);
        }
    }

    for (ItemsVector::const_iterator it = matchedItems.begin(); it != matchedItems.end(); ++it) {
        insertItemIntoTable(*it);
    }
}

void BuildWidgetTable::insertItem(FightWidgetRow * item) {
    insertItemIntoTable(item);
}

int BuildWidgetTable::insertItemIntoTable(FightWidgetRow * item) {
	
	tableWidget_->setSortingEnabled(false);

    const int targetRow = tableWidget_->rowCount();
    tableWidget_->insertRow(targetRow);

	rows_.insert(std::make_pair(item->getId(), item));

	FightWidgetRowWidget * itemWidget = new FightWidgetRowWidget(tableWidget_, item->getFortressType());

	if (item->isOwned()) {
		itemWidget->setAsGreen();
	}
	else {
		itemWidget->setAsBlue();
	}

	itemWidget->setAsBuildShell(item->isShell());
	itemWidget->setItemName(item->getNameQString());
	itemWidget->setItemId(item->getId());

	if (item->canBuild()) {
		itemWidget->setAsCanBuild(item->canBuild());
	}

	bool connected = connect(itemWidget, SIGNAL(clickedSignal(id_t)), this, SLOT(itemClickedSlot(id_t)));

	assert(connected && "Signals not connected");

	tableWidget_->setRowHeight(targetRow, itemWidget->height());
	tableWidget_->setCellWidget(targetRow, 0, itemWidget);
    
	tableWidget_->setSortingEnabled(true);

    return targetRow;
}

void BuildWidgetTable::clearFilteredContent() {
	clickedFortressRow_ = NULL;

    for (int i = tableWidget_->rowCount() - 1; i >= 0; --i) {
        tableWidget_->removeRow(i);
    }    
}

void BuildWidgetTable::removeRow(int row) {
	tableWidget_->removeRow(row);
}

void BuildWidgetTable::clearAllContent() {
    clearFilteredContent();
	rows_.clear();	
}

void BuildWidgetTable::cellClickedSlot(int row, int col) {
	FightWidgetRow * itemRow = findItemRow(row, col);

	if (itemRow == NULL) {
		KODIAK_DEBUG_COUT << "Item widget is NULL in void BuildWidgetTable::cellClickedSlot(int row, int col)";
		return;
	}

	cellClicked(itemRow, row);	
}

void BuildWidgetTable::cellClicked(FightWidgetRow * item, int row) {
	switch (tableType_) {
	case FORTRESS_BUILD_TABLE_OWNED:
		tradeWidget_->ownedFortressClicked(item, row);
		break;
	case FORTRESS_BUILD_TABLE_UNCONTROLLED:
		tradeWidget_->uncontrolledFortressClicked(item, row);
		break;
	case FORTRESS_BUILD_TABLE_CLASSES:
		tradeWidget_->fortressClassClicked(item, row);
		break;
	}
}

FightWidgetRow * BuildWidgetTable::findItemById(unsigned long id) {
    if (rows_.find(id) != rows_.end()) {
        return rows_[id];
    }
    else {
        return 0;
    }
}

FightWidgetRow * BuildWidgetTable::findItemRow(int row, int) {
	FightWidgetRowWidget * itemRow = dynamic_cast<FightWidgetRowWidget *>(tableWidget_->cellWidget(row, 0));

	if (itemRow == NULL) {
		KODIAK_DEBUG_COUT << "Item widget is NULL in FightWidgetRow * BuildWidgetTable::findItemRow(int row, int)";
		return 0;
	}

	id_t id = itemRow->getItemId();

    if (rows_.find(id) != rows_.end()) {
        return rows_[id];
    }
    else {
        return 0;
    }
}

void BuildWidgetTable::removeRowAndItem(FightWidgetRow * item, int row) {
	rows_.erase(item->getId());
	removeRow(row);
}

int BuildWidgetTable::findRowOfItem(unsigned long id) {
	int rowCount = tableWidget_->rowCount();

	for (int row = 0; row < rowCount; ++row) {
		FightWidgetRowWidget * itemRow = dynamic_cast<FightWidgetRowWidget *>(tableWidget_->cellWidget(row, 0));

		if (itemRow == NULL) {
			KODIAK_DEBUG_COUT << "Item widget is NULL in int BuildWidgetTable::findRowOfItem(unsigned long id)";
			return -1;
		}

		int inspectedId = itemRow->getItemId();

		if (inspectedId == id) {
			return row;
		}
	}
	 
	return -1;
}

void BuildWidgetTable::itemClickedSlot(id_t itemId) {
	int row = findRowOfItem(itemId);
	FightWidgetRow * itemRow = findItemById(itemId);

	if (itemRow == NULL) {
		KODIAK_DEBUG_COUT << "Item widget is NULL in void BuildWidgetTable::itemClickedSlot(id_t itemId)";
		return;
	}

	cellClicked(itemRow, row);
}


void BuildWidgetTable::setRowAsClicked(int row) {
	if (clickedFortressRow_ != NULL) {
		clickedFortressRow_->setAsClicked(false);
	}

	clickedFortressRow_ = dynamic_cast<FightWidgetRowWidget *>(tableWidget_->cellWidget(row, 0));

	if (clickedFortressRow_ == NULL) {
		return;
	}

	clickedFortressRow_->setAsClicked(true);
}

void BuildWidgetTable::setNoRowAsClicked() {
	if (clickedFortressRow_ != NULL) {
		clickedFortressRow_->setAsClicked(false);
		clickedFortressRow_ = NULL;
	}
}