#include "FightWidget.h"
#include <cassert>
#include "softProj\settings.h"
#include "FightWidgetRowWidget.h"
#include "../streams/ErrorStream.h"

extern KODIAK_DEBUG_STREAM KODIAK_DEBUG_COUT;

FortressFightTable::FortressFightTable(FightWidget * parent, EFortressFightTableType tableType) : QWidget(parent) {
    titleLabel_ = 0;
    tableWidget_ = 0;
    searchButton_ = 0;
    searchEdit_ = 0;

    tradeWidget_ = parent;	
	this->tableType_ = tableType;

	clickedFortressRow_ = NULL;
}

void FortressFightTable::setTitle(QPushButton * titleLabel) {
    this->titleLabel_ = titleLabel;
}

void FortressFightTable::setTable(QTableWidget * tableWidget) {
    this->tableWidget_ = tableWidget;
	tableWidget_->verticalHeader()->setResizeMode(QHeaderView::Interactive);
}

void FortressFightTable::setSearchButton(QPushButton * searchButton) {
    this->searchButton_ = searchButton;
}

void FortressFightTable::setShowAllButton(QPushButton * showAllButton) {
    this->showAllButton_ = showAllButton;
}

void FortressFightTable::setSearchEdit(QLineEdit * searchEdit) {
    this->searchEdit_ = searchEdit;
}

void FortressFightTable::connectSignals() {
    bool isConnected = connect(searchButton_, SIGNAL(clicked()), this, SLOT(searchClickedSlot()));
    assert(isConnected);

    isConnected = connect(showAllButton_, SIGNAL(clicked()), this, SLOT(showAllClickedSlot()));
    assert(isConnected);

    isConnected = connect(tableWidget_, SIGNAL(cellClicked(int,int)), this, SLOT(cellClickedSlot(int, int)));
    assert(isConnected);    
}

void FortressFightTable::showAllClickedSlot() {
    clearFilteredContent();

	for (std::map<long, FightWidgetRow *>::const_iterator it = rows_.begin(); it != rows_.end(); ++it) {
		insertItemIntoTable(it->second);
    }
}

void FortressFightTable::searchClickedSlot() {
    QString searchedString = searchEdit_->text();
    QRegExp rx(".*" + searchedString + ".*", Qt::CaseInsensitive);

    clearFilteredContent();

    ItemsVector matchedItems;

	for (std::map<long, FightWidgetRow *>::const_iterator it = rows_.begin(); it != rows_.end(); ++it) {
		if (rx.exactMatch(it->second->getNameQString())) {
            matchedItems.push_back(it->second);
        }
    }

    for (ItemsVector::const_iterator it = matchedItems.begin(); it != matchedItems.end(); ++it) {
        insertItemIntoTable(*it);
    }
}

FightWidgetRowWidget * FortressFightTable::insertItem(FightWidgetRow * item) {
    return insertItemIntoTable(item);
}

FightWidgetRowWidget * FortressFightTable::insertItemIntoTable(FightWidgetRow * item) {
	
	tableWidget_->setSortingEnabled(false);

    const int targetRow = tableWidget_->rowCount();
    tableWidget_->insertRow(targetRow);

	rows_.insert(std::make_pair(item->getId(), item));

	FightWidgetRowWidget * itemWidget = new FightWidgetRowWidget(tableWidget_, item->getFortressType());

	if (item->isOwned()) {
		itemWidget->setAsGreen();
	}
	else {
		itemWidget->setAsBlue();
	}

	if (item->getShellIconEnabled()) {
		itemWidget->setAsBuildShell(item->isShell());
	}

	itemWidget->setItemName(item->getNameQString());
	itemWidget->setItemId(item->getId());

	bool connected = true;
	connected = connected && connect(itemWidget, SIGNAL(clickedSignal(id_t)), this, SLOT(itemClickedSlot(id_t)));
	connected = connected && connect(itemWidget, SIGNAL(doubleClickedSignal(id_t)), this, SLOT(itemDoubleClickedSlot(id_t)));
	assert(connected && "Signals not connected");

	tableWidget_->setRowHeight(targetRow, itemWidget->height());
	tableWidget_->setCellWidget(targetRow, 0, itemWidget);
    
	tableWidget_->setSortingEnabled(true);

	return itemWidget;
}

void FortressFightTable::clearFilteredContent() {
	clickedFortressRow_ = NULL;

    for (int i = tableWidget_->rowCount() - 1; i >= 0; --i) {
        tableWidget_->removeRow(i);
    }    
}

void FortressFightTable::removeRow(int row) {
	tableWidget_->removeRow(row);
}

void FortressFightTable::clearAllContent() {
    clearFilteredContent();
	rows_.clear();
}

void FortressFightTable::cellClickedSlot(int row, int col) {
	FightWidgetRow * itemRow = findItemRow(row, col);

	if (itemRow == NULL) {
		KODIAK_DEBUG_COUT << "Item widget is NULL in void FortressFightTable::cellClickedSlot(int row, int col)";
		return;
	}

	cellClicked(itemRow, row);
}

void FortressFightTable::cellClicked(FightWidgetRow * item, int row) {
	switch (tableType_) {
	case FORTRESS_FIGHT_TABLE_OWNED:
		tradeWidget_->ownedFortressClicked(item, row);
		break;
	case FORTRESS_FIGHT_TABLE_ENEMY:
		tradeWidget_->enemyFortressClicked(item, row);
		break;
	}
}

void FortressFightTable::cellDoubleClicked(FightWidgetRow * item, int row) {
	switch (tableType_) {
	case FORTRESS_FIGHT_TABLE_OWNED:
		tradeWidget_->ownedFortressDoubleClicked(item, row);
		break;
	case FORTRESS_FIGHT_TABLE_ENEMY:
		tradeWidget_->enemyFortressDoubleClicked(item, row);
		break;
	}
}

FightWidgetRow * FortressFightTable::findItemById(id_t id) {
    if (rows_.find(id) != rows_.end()) {
        return rows_[id];
    }
    else {
        return 0;
    }
}

FightWidgetRowWidget * FortressFightTable::findRowWidgetById(id_t id) {
	int rowCount = tableWidget_->rowCount();

	for (int row = 0; row < rowCount; ++row) {
		FightWidgetRowWidget * itemRow = dynamic_cast<FightWidgetRowWidget *>(tableWidget_->cellWidget(row, 0));

		if (itemRow == NULL) {
			KODIAK_DEBUG_COUT << "Item widget is NULL in int FortressFightTable::findRowWidgetById(id_t id)";
			return NULL;
		}

		if (itemRow->getItemId() == id) {
			return itemRow;
		}
	}

	return NULL;
}

FightWidgetRow * FortressFightTable::findItemRow(int row, int) {
	FightWidgetRowWidget * itemRow = dynamic_cast<FightWidgetRowWidget *>(tableWidget_->cellWidget(row, 0));

	if (itemRow == NULL) {
		KODIAK_DEBUG_COUT << "Item widget is NULL in FightWidgetRow * FortressFightTable::findItemRow(int row, int)";
		return NULL;
	}

	id_t id = itemRow->getItemId();

    if (rows_.find(id) != rows_.end()) {
        return rows_[id];
    }
    else {
        return NULL;
    }
}

void FortressFightTable::removeRowAndItem(FightWidgetRow * item, int row) {
	rows_.erase(item->getId());
	removeRow(row);
}

int FortressFightTable::findRowOfItem(unsigned long id) {
	int rowCount = tableWidget_->rowCount();

	for (int row = 0; row < rowCount; ++row) {
		FightWidgetRowWidget * itemRow = dynamic_cast<FightWidgetRowWidget *>(tableWidget_->cellWidget(row, 0));

		if (itemRow == NULL) {
			KODIAK_DEBUG_COUT << "Item widget is NULL in int FortressFightTable::findRowOfItem(unsigned long id)";
			return -1;
		}

		int inspectedId = itemRow->getItemId();

		if (inspectedId == id) {
			return row;
		}
	}
	 
	return -1;
}

void FortressFightTable::itemClickedSlot(id_t itemId) {
	int row = findRowOfItem(itemId);

	FightWidgetRow * item = findItemById(itemId);

	if (item == NULL) {
		KODIAK_DEBUG_COUT << "Item widget is NULL in void FortressFightTable::itemClickedSlot(id_t itemId)";
		return;
	}

	cellClicked(item, row);
}

void FortressFightTable::itemDoubleClickedSlot(id_t itemId) {
	int row = findRowOfItem(itemId);

	FightWidgetRow * item = findItemById(itemId);

	if (item == NULL) {
		KODIAK_DEBUG_COUT << "Item widget is NULL in void FortressFightTable::itemDoubleClickedSlot(id_t itemId)";
		return;
	}

	cellDoubleClicked(item, row);
}

void FortressFightTable::setNoRowAsClicked() {
	if (clickedFortressRow_ != NULL) {
		clickedFortressRow_->setAsClicked(false);
		clickedFortressRow_ = NULL;
	}
}

void FortressFightTable::setRowAsClicked(int row) {
	setNoRowAsClicked();

	clickedFortressRow_ = dynamic_cast<FightWidgetRowWidget *>(tableWidget_->cellWidget(row, 0));

	if (clickedFortressRow_ == NULL) {
		return;
	}

	clickedFortressRow_->setAsClicked(true);
}

FightWidgetRowWidget * FortressFightTable::getClickedRow() {
	return clickedFortressRow_;
}