
#include <exception>
#include <math.h>
#include "datetime.h"
#include <windows.h>

using namespace dv_time;

// ------------------------------------------------------------------------------------------------

datetime :: datetime()
{
	datetime_ = 0;
}

datetime :: datetime(double dt)
{
	datetime_ = dt;
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: operator=(const datetime& other)
{
	datetime_ = other.datetime_;
	return *this;
}

// ------------------------------------------------------------------------------------------------

const datetime datetime :: operator+(const datetime& other) const
{
	return datetime(datetime_ + other.datetime_);
}

// ------------------------------------------------------------------------------------------------

const datetime datetime :: operator*(const double other) const
{
	return datetime(datetime_ * other);
}

// ------------------------------------------------------------------------------------------------

const datetime datetime :: operator*(const unsigned long long other) const
{
	return datetime(datetime_ * (double)other);
}

// ------------------------------------------------------------------------------------------------

const datetime datetime :: operator/(const double other) const
{
	return datetime(datetime_ / other);
}

// ------------------------------------------------------------------------------------------------

const datetime datetime :: operator/(const unsigned long long other) const
{
	return datetime(datetime_ / (double)other);
}


// ------------------------------------------------------------------------------------------------

const datetime datetime :: operator-(const datetime& other) const
{
	return datetime(datetime_ - other.datetime_);
}

// ------------------------------------------------------------------------------------------------

const double datetime :: operator/(const datetime& other) const
{
	return (datetime_ / other.datetime_);
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: operator+=(const datetime& other)
{
	datetime_ += other.datetime_;
	return *this;
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: operator-=(const datetime& other)
{
	datetime_ -= other.datetime_;
	return *this;
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: operator*=(const datetime& other)
{
	datetime_ *= other.datetime_;
	return *this;
}

// ------------------------------------------------------------------------------------------------

bool datetime :: operator==(const datetime& other) const
{
	return datetime_ == other.datetime_;
}

// ------------------------------------------------------------------------------------------------

bool datetime :: operator!=(const datetime& other) const
{
	return ! (*this == other);
}

// ------------------------------------------------------------------------------------------------

datetime :: operator double() const
{
	return datetime_;
}

// ------------------------------------------------------------------------------------------------

const datetime datetime :: second(void)
{
	return datetime((double)datetime :: minute() / 60.0);
}

// ------------------------------------------------------------------------------------------------

const datetime datetime :: nanosecond(void)
{
	return datetime((double)datetime :: microsecond() / 1000.0);
}

// ------------------------------------------------------------------------------------------------

const datetime datetime :: microsecond(void)
{
	return datetime((double)datetime :: millisecond() / 1000.0);
}

// ------------------------------------------------------------------------------------------------

const datetime datetime :: millisecond(void)
{
	return datetime((double)datetime :: second() / 1000.0);
}

// ------------------------------------------------------------------------------------------------

const datetime datetime :: minute(void)
{
	return datetime((double)datetime :: hour() / 60.0);
}

// ------------------------------------------------------------------------------------------------

const datetime datetime :: hour(void)
{
	return datetime((double)datetime :: day() / 24.0);
}

// ------------------------------------------------------------------------------------------------

const datetime datetime :: day(void)
{
	return datetime(1.0);
}

// ------------------------------------------------------------------------------------------------

const datetime datetime :: month(void)
{
	return datetime((double)datetime :: day() * 31.0);
}

// ------------------------------------------------------------------------------------------------

const datetime datetime :: year(void)
{
	return datetime((double)datetime :: day() * 365.0);
}

// ------------------------------------------------------------------------------------------------

static const int monthDays[]		  = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
static const int monthDaysLeapYear[]  = {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
static const int dateDelta			  = 693594; // dny mezi 1.1.0001 a 31.12.1899
static const int mSecPerDay			  = (int) (datetime :: day() / datetime :: millisecond());
static const int mSecPerSecond  	  = (int) (datetime :: second() / datetime :: millisecond());
static const int secondPerMinute	  = (int) (datetime :: minute() / datetime :: second());
static const int minutePerHour	      = (int) (datetime :: hour() / datetime :: minute());

// ------------------------------------------------------------------------------------------------

bool dv_time :: isLeapYear(int year)
{
	return (year % 4 == 0) && ((year % 100 != 0) || (year % 400 == 0));
}

// ------------------------------------------------------------------------------------------------

void dv_time :: decodeTime(const datetime& t, int& hours, int& minutes, int& seconds, int& milliseconds)
{
	double time = ((double)t - floor((double)t)) * mSecPerDay;
	int tval = (int) time;

	milliseconds = tval % 1000;		tval /= 1000;
	seconds		 = tval % 60;		tval /= 60;
	minutes		 = tval % 60;		tval /= 60;
	hours		 = tval;
}

// ------------------------------------------------------------------------------------------------

void dv_time :: decodeDate(const datetime& dt, int& year, int& month, int& day)
{
	int t = (int)((double)dt + dateDelta);

	const int D1   = 365;
    const int D4   = D1 * 4 + 1;
    const int D100 = D4 * 25 - 1;
    const int D400 = D100 * 4 + 1;

	t--;
    year = 1;
    while (t >= D400) 
	{
      t    -= D400;
      year += 400;
	}

	int i, d;
	i = t / D100;
	d = t % D100;

	if (i == 4)
	{
		i--;
		d += D100;
	}

	year += i * 100;

	i = d / D4;
	d = d % D4;

	year += i * 4;

	i = d / D1;
	d = d % D1;

	if (i == 4)
	{
		i--;
		d += D1;
	}

	year += i;

	const int* mDays = isLeapYear(year) ? monthDaysLeapYear : monthDays;

    month = 1;
    while (true) 
	{
      i = mDays[month-1];
      if (d < i) 
	  {
		  break;
	  }

      d -= i;
      month++;
	}

	day = d + 1;
}

// ------------------------------------------------------------------------------------------------

void dv_time :: decodeDateTime(const datetime& t, int& year, int& month, int& day, 
		int& hours, int& minutes, int& seconds, int& milliseconds)
{
	decodeTime(t, hours, minutes, seconds, milliseconds);
	decodeDate(t, year, month, day);
}

// ------------------------------------------------------------------------------------------------

datetime dv_time :: encodeDateTime(int year, int month, int day, int hours, int minutes, int seconds, int milliseconds)
{
	return encodeDate(year, month, day) + encodeTime(hours, minutes, seconds, milliseconds);
}

// ------------------------------------------------------------------------------------------------

datetime dv_time :: encodeSeconds(unsigned long long seconds)
{
	unsigned long long dd =      ((seconds / (24 * 60 * 60 * 1))       );
	               int hh = (int)((seconds / (     60 * 60 * 1)) % (24));
				   int mm = (int)((seconds / (          60 * 1)) % (60));
				   int ss = (int)((seconds / (               1)) % (60));
				   int ms =                                          0  ;
	
	return dd + encodeTime(hh, mm, ss, ms);
}

// ------------------------------------------------------------------------------------------------

datetime dv_time :: encodeTime(int hours, int minutes, int seconds, int milliseconds)
{
	if ((hours < 24) && (minutes < 60) && (seconds < 60) && (milliseconds < 1000))
	{
		return hours       *  (double) datetime :: hour()  
			+ minutes      *  (double) datetime :: minute()
			+ seconds      *  (double) datetime :: second() 
			+ milliseconds *  (double) datetime :: millisecond();
	}
	else 
	{
  	    throw std :: exception("Bad date time values");
	}
}

// ------------------------------------------------------------------------------------------------

datetime dv_time :: encodeDate(int year, int month, int day)
{
	const int* mDays = isLeapYear(year) ? monthDaysLeapYear : monthDays;
	double value, oneday;
	value = 0;

	// zkontroluj, zda byly zadany spravne hodnoty, pokud ano, enkoduj datum
    if ((year >= 1) && (year <= 9999) && (month >= 1) && (month <= 12) && (day >= 1) && (day <= mDays[month-1]))
    {
		// pridej dny z tohoto roku
		oneday = (double) datetime :: day();
		for (int i = 1; i < month; i++)
		{
			value += mDays[i-1] * oneday;
		}

		int ym1 = year - 1;
		return value + ym1 * (double) datetime :: year() + (ym1 / 4 - ym1 / 100 + ym1 / 400 - dateDelta + day) * oneday;
    }
    else
    {
  	    throw std :: exception("Bad date time values");
    }
}

// ------------------------------------------------------------------------------------------------

datetime dv_time :: datediff(const datetime& d1, const datetime& d2)
{
	if (d1 < d2)
	{
		return d2 - d1;
	}
	else
	{
		return d1 - d2;
	}
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: yearsBetween(const datetime& d1, const datetime& d2)
{
	return (long long)(datediff(d1, d2) / datetime :: year());
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: monthsBetween(const datetime& d1, const datetime& d2)
{
	return (long long)(datediff(d1, d2) / datetime :: month());
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: daysBetween(const datetime& d1, const datetime& d2)
{
	return (long long)(datediff(d1, d2) / datetime :: day());
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: hoursBetween(const datetime& d1, const datetime& d2)
{
	return (long long)(datediff(d1, d2) / datetime :: hour());
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: minutesBetween(const datetime& d1, const datetime& d2)
{
	return (long long)(datediff(d1, d2) / datetime :: minute());
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: secondsBetween(const datetime& d1, const datetime& d2)
{
	return (long long)(datediff(d1, d2) / datetime :: second());
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: millisecondsBetween(const datetime& d1, const datetime& d2)
{
	return (long long)(datediff(d1, d2) / datetime :: millisecond());
}

// ------------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------------

long long dv_time :: yearsBegunBetween(const datetime& d1, const datetime& d2)
{
	return (long long)ceil((datediff(d1, d2) / datetime :: year()));
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: monthsBegunBetween(const datetime& d1, const datetime& d2)
{
	return (long long)ceil((datediff(d1, d2) / datetime :: month()));
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: daysBegunBetween(const datetime& d1, const datetime& d2)
{
	return (long long)ceil((datediff(d1, d2) / datetime :: day()));
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: hoursBegunBetween(const datetime& d1, const datetime& d2)
{
	return (long long)ceil((datediff(d1, d2) / datetime :: hour()));
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: minutesBegunBetween(const datetime& d1, const datetime& d2)
{
	return (long long)ceil((datediff(d1, d2) / datetime :: minute()));
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: secondsBegunBetween(const datetime& d1, const datetime& d2)
{
	return (long long)ceil((datediff(d1, d2) / datetime :: second()));
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: millisecondsBegunBetween(const datetime& d1, const datetime& d2)
{
	return (long long)ceil((datediff(d1, d2) / datetime :: millisecond()));
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: yearsOf(const datetime& dt)
{
	int y,m,d;
	decodeDate(dt, y, m, d);
	return y;
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: monthsOf(const datetime& dt)
{
	int y,m,d;
	decodeDate(dt, y, m, d);
	return m;
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: daysOf(const datetime& dt)
{
	int y,m,d;
	decodeDate(dt, y, m, d);
	return d;
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: hoursOf(const datetime& dt)
{
	int h,m,s,ms;
	decodeTime(dt, h, m, s, ms);
	return h;
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: minutesOf(const datetime& dt)
{
	int h,m,s,ms;
	decodeTime(dt, h, m, s, ms);
	return m;
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: secondsOf(const datetime& dt)
{
	int h,m,s,ms;
	decodeTime(dt, h, m, s, ms);
	return s;
}

// ------------------------------------------------------------------------------------------------

long long dv_time :: millisecondsOf(const datetime& dt)
{
	int h,m,s,ms;
	decodeTime(dt, h, m, s, ms);
	return ms;
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: incYear(int years)
{
	datetime_ += years * datetime :: year();
	return *this;
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: incMonth(int months)
{
	datetime_ += months * datetime :: month();
	return *this;
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: incDay(int days)
{
	datetime_ += days * datetime :: day();
	return *this;
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: incHour(int hours)
{
	datetime_ += hours * datetime :: hour();
	return *this;
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: incMinute(int minutes)
{
	datetime_ += minutes * datetime :: minute();
	return *this;
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: incSecond(int seconds)
{
	datetime_ += seconds * datetime :: second();
	return *this;
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: incMillisecond(int milliseconds)
{
	datetime_ += milliseconds * datetime :: millisecond();
	return *this;
}

// ------------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------------

datetime& datetime :: decYear(int years)
{
	datetime_ -= years * datetime :: year();
	return *this;
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: decMonth(int months)
{
	datetime_ -= months * datetime :: month();
	return *this;
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: decDay(int days)
{
	datetime_ -= days * datetime :: day();
	return *this;
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: decHour(int hours)
{
	datetime_ -= hours * datetime :: hour();
	return *this;
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: decMinute(int minutes)
{
	datetime_ -= minutes * datetime :: minute();
	return *this;
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: decSecond(int seconds)
{
	datetime_ -= seconds * datetime :: second();
	return *this;
}

// ------------------------------------------------------------------------------------------------

datetime& datetime :: decMillisecond(int milliseconds)
{
	datetime_ -= milliseconds * datetime :: millisecond();
	return *this;
}

// ------------------------------------------------------------------------------------------------

bool datetime :: operator<(const datetime& other) const
{
	return datetime_ < other.datetime_;
}

// ------------------------------------------------------------------------------------------------

bool datetime :: operator>(const datetime& other) const
{
	return datetime_ > other.datetime_;
}

// ------------------------------------------------------------------------------------------------

bool datetime :: operator<=(const datetime& other) const
{
	return datetime_ <= other.datetime_;
}

// ------------------------------------------------------------------------------------------------

bool datetime :: operator>=(const datetime& other) const
{
	return datetime_ >= other.datetime_;
}

// ------------------------------------------------------------------------------------------------

datetime dv_time :: now(void)
{
	SYSTEMTIME t;
	GetSystemTime(&t);
	return encodeDateTime(t.wYear, t.wMonth, t.wDay, t.wHour, t.wMinute, t.wSecond, t.wMilliseconds);
}

// ------------------------------------------------------------------------------------------------

#define SETIFNOTNULL(pointer, value) if (pointer != NULL) {*pointer = value;}

// ------------------------------------------------------------------------------------------------

void dv_time :: decodeTime(const datetime* t, int* hours, int* minutes, int* seconds, int* milliseconds)
{
	int h,m,s,ms;
	decodeTime(*t, h, m, s, ms);

	SETIFNOTNULL(hours, h);
	SETIFNOTNULL(minutes, m);
	SETIFNOTNULL(seconds, s);
	SETIFNOTNULL(milliseconds, ms);
}

// ------------------------------------------------------------------------------------------------

void dv_time :: decodeDate(const datetime* t, int* year, int* month, int* day)
{
	int y, m, d;
	decodeDate(*t, y, m, d);

	SETIFNOTNULL(year, y);
	SETIFNOTNULL(month, m);
	SETIFNOTNULL(day, d);
}

// ------------------------------------------------------------------------------------------------

void dv_time :: decodeDateTime(const datetime* t, int* year, int* month, int* day, 
		int* hours, int* minutes, int* seconds, int* milliseconds)
{
	decodeTime(t, hours, minutes, seconds, milliseconds);
	decodeDate(t, year, month, day);
}

// ------------------------------------------------------------------------------------------------